#' Graph spectral entropy
#'
#' Given an adjacency matrix A, it returns the spectral entropy of the 
#' corresponding graph.
#' @param A a symmetric matrix of nonnegative real values. For unweighted graphs 
#' it contains only 0s and 1s and corresponds to the adjacency matrix. For 
#' weighted graphs, it may contain any nonnegative real value and corresponds to
#' the weighted adjancency matrix.
#' @param bandwidth string indicating which criterion should be used
#' to choose the bandwidth for the spectral density estimation. The available 
#' criteria are "Silverman" (default) and "Sturges".
#'
#' @return a real number corresponding to the graph spectral entropy.  
#'
#' @references
#' Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
#' André Fujita. Discriminating Different Classes of Biological Networks by 
#' Analyzing the Graph Spectra Distribution. PLoS ONE 7, no. 12 
#' (December 2012): e49949. doi:10.1371/journal.pone.0049949.
#' 
#' @examples
#' library(igraph)
#' G <- erdos.renyi.game(100, p=0.5)
#' A <- as.matrix(get.adjacency(G))
#' entropy <- graph.entropy(A)
#' entropy
#'
#' @export

graph.entropy <- function(A, bandwidth="Silverman") {
    f <- spectralDensity(A, bandwidth=bandwidth)
    if (sum(is.na(f)) > 0)
        return(NA)
    y <- f$y
    n <- length(y)
    i <- which(y != 0)
    y[i] <- y[i]*log(y[i])
    return(-trapezoidSum(f$x, y))
}

#' Graph Information Criterion (GIC)
#'
#' Returns the Kullback-Leibler divergence between an observed graph and a given 
#' graph model. 
#'
#' @param A a symmetric matrix of nonnegative real values. For unweighted graphs 
#' it contains only 0s and 1s and corresponds to the adjacency matrix. For 
#' weighted graphs, it may contain any nonnegative real value and corresponds to
#' the weighted adjancency matrix.
#' @param model either a list, a string or a function describing a graph model: 
#' 
#' A list that represents the spectral density of a model. It contains the 
#' components "x" and "y", which are numeric vectors of the same size. The "x" 
#' component contains the points at which the density was computed and the "y" 
#' component contains the observed density.
#'
#' A string that indicates one of the following models: "ER" (Erdos-Renyi random 
#' graph), "GRG" (geometric random graph), "KR" (k regular 
#' random graph), "WS" (Watts-Strogatz model), and "BA" (Barabási-Albert model).
#' When the argument model is a string, the user must also provide the model 
#' parameter by using the argument p.  
#'
#' A function that returns a graph according to a graph model. It must have two 
#' arguments: n (graph size) and p (a model parameter). The model parameter 
#' will be provided by the argument p of the GIC function.
#' @param p the model parameter. The user must provide a valid parameter if the
#' argument "model" is a string or a function. For the "ER", "GRG", "KR", "WS", 
#' and "BA" models, the parameter p corresponds to the probability to connect a 
#' pair of vertices, the radius used to contruct the geometric graph in a unit 
#' square, the degree k of a regular graph, the probability to reconnect a 
#' vertex, and the scale exponent, respectively.
#' @param bandwidth string indicating which criterion should be used
#' to choose the bandwidth for the spectral density estimation. The available 
#' criteria are "Silverman" (default) and "Sturges".
#'
#' @return A real number corresponding to the Kullback-Leibler divergence 
#' between the observed graph and a given model.
#'
#' @references
#' Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
#' André Fujita. Discriminating Different Classes of Biological Networks by 
#' Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
#' (December 2012): e49949. doi:10.1371/journal.pone.0049949.
#' 
#' @examples
#' library(igraph)
#' A <- as.matrix(get.adjacency(erdos.renyi.game(100, p=0.5)))
#' # Using a string to indicate the graph model
#' result1 <- GIC(A, "ER", 0.5)
#' result1
#'
#' # Using a function to describe the graph model
#' # Erdos-Renyi graph
#' model <- function(n, p) {
#'    return(as.matrix(get.adjacency(erdos.renyi.game(n, p))))
#' }
#' result2 <- GIC(A, model, 0.5)
#' result2
#' @export

GIC <- function(A, model, p=NULL, bandwidth="Silverman") {
    if (class(model) == "list") {
        f2 <- model
        f1 <- spectralDensity(A, from=min(f2$x), to=max(f2$x), 
                              bandwidth=bandwidth, npoints=length(f2$x))
    }
    else {
        fun <- model
        if (class(model) == "character") {
            if (model == "WS")
                fun <- WSfun(as.integer(sum(A)/(2*ncol(A))))
            else fun <- matchFunction(model)
        }
        eigenvalues <- (as.numeric(eigen(A, only.values = TRUE)$values)/
                        sqrt(nrow(A)))
        f2 <- modelSpectralDensity(fun, ncol(A), p, from=min(eigenvalues), 
                                   to=max(eigenvalues), bandwidth=bandwidth,
                                   npoints=1024)
        f1 <- gaussianDensity(eigenvalues, from=min(f2$x), to=max(f2$x), 
                              bandwidth=bandwidth, npoints=1024)
    }
    if (sum(is.na(f1)) > 0)
        return(NA)
    if (sum(is.na(f2)) > 0)
        return(NA)
    return(KL(f1, f2))
}


#' Graph parameter estimator
#'
#' Estimates the model parameter that best approximates the model to the 
#' observed graph. It can take a long time to execute, specially for the k 
#' regular graph model.
#'
#' @param A a symmetric matrix of nonnegative real values. For unweighted graphs 
#' it contains only 0s and 1s and corresponds to the adjacency matrix. For 
#' weighted graphs, it may contain any nonnegative real value and corresponds to
#' the weighted adjancency matrix.
#' @param model either a string or a function describing a graph model: 
#' 
#' A string that indicates one of the following models: "ER" (Erdos-Renyi random 
#' graph), "GRG" (geometric random graph), "KR" (k regular 
#' random graph), "WS" (Watts-Strogatz model), and "BA" (Barabási-Albert model).
#'
#' A function that returns a graph according to a graph model. It must have two 
#' arguments: n (graph size) and p (a model parameter).
#' @param parameters numeric vector containing the values that will be 
#' considerated for the model parameter. For the "ER", "GRG", "KR", "WS", and 
#' "BA" models, the parameter corresponds to the probability to connect a pair 
#' of vertices, the radius used to contruct the geometric graph in a unit 
#' square, the degree k of a regular graph, the probability to reconnect a 
#' vertex, and the scale exponent, respectively. For those models there are 
#' default parameter values that will be considered, and then it is optional to 
#' use this argument. However if the user provides a custom function to generate 
#' graphs, then it is also necessary to provide the values that will be 
#' considered for the estimation. 
#' @param bandwidth string indicating which criterion should be used
#' to choose the bandwidth for the spectral density estimation. The available 
#' criteria are "Silverman" (default) and "Sturges".
#' @param eps precision of the grid (default is 0.01)

#' @return A list containing:
#' \item{p}{the parameter estimate. For the "ER", "GRG", "KR", "WS", and "BA"
#' models, the parameter corresponds to the probability to connect a pair of 
#' vertices, the radius used to contruct the geometric graph in a unit square,
#' the degree k of a regular graph, the probability to reconnect a vertex, and 
#' the scale exponent, respectively.}  
#' \item{KL}{the Kullback-Leibler divergence between the observed graph and the 
#' model obtained by the estimator.}
#'
#' @references
#' Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
#' André Fujita. Discriminating Different Classes of Biological Networks by 
#' Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
#' (December 2012): e49949. doi:10.1371/journal.pone.0049949.
#' 
#' @examples
#' library(igraph)
#' A <- as.matrix(get.adjacency(erdos.renyi.game(100, p=0.5)))
#'
#' # Using a string to indicate the graph model
#' result1 <- graph.param.estimator(A, "ER")
#' result1
#'
#' # Using a function to describe the graph model
#' # Erdos-Renyi graph
#' model <- function(n, p) {
#'    return(as.matrix(get.adjacency(erdos.renyi.game(n, p))))
#' }
#' result2 <- graph.param.estimator(A, model,  seq(0, 1, 0.01))
#' result2
#'
#' @export

graph.param.estimator <- function(A, model, parameters=NULL, 
                                  bandwidth="Silverman", eps=0.01) {
    n <- ncol(A)
    if (class(model) == "function" && is.null(parameters)) {
        stop("It is necessary to enter the paremeters that will be evaluated.")
    }
    if (is.null(parameters)) {
            parameters <- seq(0, 1, eps)
            if (model == "GRG")
                parameters <- seq(0, sqrt(2), eps)
            if (model == "BA")
                parameters <- seq(1, 4, eps)
            if (model == "KR")
                parameters <- as.integer(seq(0, 1, eps)*n)
    }
    pmin <- -1
    klmin <- Inf
    for (p in parameters) {
       kl <- GIC(A, model, p, bandwidth) 
       if (kl < klmin) {
           klmin <- kl
           pmin <- p
       }
    }
    return(list("p"=pmin, "KL"=klmin))
}



#' Graph model selection
#'
#' Selects the graph model that best approximates the observed graph. It can 
#' take a long time to execute, specially for the k regular graph model.
#'
#' @param A a symmetric matrix of nonnegative real values. For unweighted graphs 
#' it contains only 0s and 1s and corresponds to the adjacency matrix. For 
#' weighted graphs, it may contain any nonnegative real value and corresponds to
#' the weighted adjancency matrix.
#' @param models either a character vector or a list of functions describing 
#' graph models: 
#' 
#' A character vector cotaining some of the following models: "ER" (Erdos-Renyi 
#' random graph), "GRG" (geometric random graph), "KR" (k regular random graph), 
#' "WS" (Watts-Strogatz model), and "BA" (Barabási-Albert model).
#'
#' A list of functions. Each function returns a graph according to a graph model
#' and has two arguments: n (graph size) and p (a model parameter), in this 
#' order.
#' 
#' If the argument "models" is NULL, then the "ER", "WS", and "BA" models will 
#' be considered.
#' @param parameters list of numeric vectors. Each vector contains the values 
#' that will be considerated for the parameter of the corresponding model.
#' For the "ER", "GRG", "KR", "WS", and "BA" models, the parameter corresponds 
#' to the probability to connect a pair of vertices, the radius used to contruct 
#' the geometric graph in a unit square, the degree k of a regular graph, the 
#' probability to reconnect a vertex, and the scale exponent, respectively. For 
#' those models there are default parameter values that will be considered, and 
#' then it is optional to use this argument. However if the user provides custom 
#' functions to generate graphs, then it is also necessary to provide the values 
#' that will be considered for the parameter estimation. 
#' @param bandwidth string indicating which criterion should be used
#' to choose the bandwidth for the spectral density estimation. The available 
#' criteria are "Silverman" (default) and "Sturges".
#' @return A matrix in which each row corresponds to a model, the column "p"
#' corresponds to the parameter estimate, and the column "KL" corresponds to
#' the Kullback-Leibler divergence between the observed graph and the model in 
#' the row. 
#' @references
#' Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
#' André Fujita. Discriminating Different Classes of Biological Networks by 
#' Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
#' (December 2012): e49949. doi:10.1371/journal.pone.0049949.
#' 
#' @examples
#' library(igraph)
#' A <- as.matrix(get.adjacency(erdos.renyi.game(100, p=0.5)))
#'
#' # Using the default graph models (ER, WS and BA)
#' result1 <- graph.model.selection(A)
#' result1
#'
#' # Using strings to indicate the graph models
#' result2 <- graph.model.selection(A, models=c("ER", "WS", "BA"))
#' result2
#'
#' # Using functions to describe the graph models
#' # Erdos-Renyi graph
#' model1 <- function(n, p) {
#'    return(as.matrix(get.adjacency(erdos.renyi.game(n, p))))
#' }
#' # Watts-Strougatz graph
#' model2 <- function(n, pr, K=8) {
#'     return(as.matrix(igraph::get.adjacency(igraph::watts.strogatz.game(1, n, 
#'                                              K, pr))))
#'}
#' # Barabasi-Albert graph
#' model3 <- function(n, ps) {
#'     return(as.matrix(igraph::get.adjacency(igraph::barabasi.game(n, power=ps,
#'                                                 directed = FALSE))))
#'}
#' parameters <- list(seq(0, 1, 0.01), seq(0, 1, 0.01), seq(1, 4, 0.01))
#' result3 <- graph.model.selection(A, list(model1, model2, model3), parameters)
#' result3
#' @export

graph.model.selection <- function(A, models=NULL, parameters=NULL, 
                                  bandwidth="Silverman") {
    n <- ncol(A)
    if (class(models) == "list" && is.null(parameters)) {
        stop("It is necessary to enter the paremeters that will be evaluated.")
    }
    if (is.null(models)) {
        models <- c("ER", "WS", "BA")
    }
    results <- matrix(NA, length(models), 2)
    colnames(results) <- c("p", "KL")
    if (class(models) == "character") {
        rownames(results) <- models
    }
    p <- NULL
    for (i in 1:length(models)) {
        if (!is.null(parameters))
            p <- parameters[[i]]
        r <- graph.param.estimator(A, models[[i]], p, bandwidth) 
        results[i, "p"] <- r$p
        results[i, "KL"] <- r$KL
    }
    return(results)
}


#' Bootstrap test for the graph Jensen-Shannon (JS) divergence.
#'
#' Given two lists of graphs, x and y, it tests H0: "JS divergence between x and  
#' y is 0" against H1: "JS divergence between x and y is larger than 0".
#'
#' @param x a list of symmetric matrices with nonnegative real 
#' entries. For unweighted graphs, each matrix contains only 0s and 1s and 
#' corresponds to an adjacency matrix. For weighted graphs, each matrix may 
#' contain any nonnegative real value and corresponds to the weighted adjancency 
#' matrix.
#' @param y a list of symmetric matrices with nonnegative real 
#' entries. For unweighted graphs, each matrix contains only 0s and 1s and 
#' corresponds to an adjacency matrix. For weighted graphs, each matrix may 
#' contain any nonnegative real value and corresponds to the weighted adjancency 
#' matrix.
#' @param numBoot integer indicating the number of bootstrap resamplings.
#' @param bandwidth character string indicating which criterion should be used
#' to choose the bandwidth for the spectral density estimation. The available 
#' criteria are "Silverman" (default) and "Sturges".
#'
#' @return A list containing:
#' \item{JS}{the Jensen-Shannon divergence between the two graph sets.}  
#' \item{pvalue}{the p-value of the test.}
#'
#' @references
#' Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
#' André Fujita. Discriminating Different Classes of Biological Networks by 
#' Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
#' (December 2012): e49949. doi:10.1371/journal.pone.0049949.
#' 
#' @examples
#' library(igraph)
#' x <- y <- list()
#' for (i in 1:40)
#'    x[[i]] <- as.matrix(get.adjacency(erdos.renyi.game(50, p=0.5)))
#' for (i in 1:45)
#'    y[[i]] <- as.matrix(get.adjacency(erdos.renyi.game(50, p=0.2)))
#' 
#' result <- graph.test(x, y)
#' result
#'
#' @export

graph.test <- function(x, y, numBoot=1000, bandwidth="Silverman") {
    adjacencyMatrices <- append(x, y)
    labels <- c(rep(0, length(x)), rep(1, length(y)))
    f <- nSpectralDensities(adjacencyMatrices)
    densities <- f$densities
    x <- f$x
    y1 <- rowMeans(densities[, labels==0])
    y2 <- rowMeans(densities[, labels==1])
    n1 <- length(which(labels==0))
    n2 <- length(which(labels==1))
    results <- vector(len=numBoot)
    ngraphs <- length(adjacencyMatrices)
    result <- JS(list("x"=x, "y"=y1), list("x"=x, "y"=y2))
    for (i in 1:numBoot) {
        b1 <- sample(1:ngraphs, n1, replace=T)
        b2 <- sample(1:ngraphs, n2, replace=T)
        y1 <- rowMeans(densities[, b1])
        y2 <- rowMeans(densities[, b2])
        results[i] <- JS(list("x"=x, "y"=y1), list("x"=x, "y"=y2))
    }
    pvalue <- (sum(results >= result))/numBoot
    return(list("JS"=result, "pvalue"=pvalue))
}


#' GANOVA Graph Analysis of Variability
#'
#' This function statistically tests whether two or more sets of graphs are generated by the same
#' random graph model. It is a generalization of the graph.test function.
#' @param x a list of adjacency matrices of graphs
#' @param labels an array of integers indicating the labels of each graph
#' @param maxBoot the number of bootstrap samples to be generated
#'
#' @return pvalue the p-value of the test
#'
#' @examples
#' library(igraph)
#' tamGraph1 <- 50
#' tamGraph2 <- 50
#' tamGraph3 <- 50
#' maxGraphs1 <- maxGraphs2 <- maxGraphs3 <- 50
#' p1 <- rnorm(maxGraphs1, 0, 1)        ## ER graphs with p~N(0,1)
#' p2 <- rnorm(maxGraphs2, 0.7, 1)      ## ER graphs with p~N(0.7,1)
#' p3 <- rnorm(maxGraphs3, 0, 1)        ## ER graphs with p~N(0,1)
#' tmp <- c(p1, p2, p3)
#' p1 <- (p1 - min(tmp)) / (max(tmp) - min(tmp))
#' p2 <- (p2 - min(tmp)) / (max(tmp) - min(tmp))
#' p3 <- (p3 - min(tmp)) / (max(tmp) - min(tmp))
#' g.adj <- list()
#' cont <- 1
#' for (sim in 1:maxGraphs1) {
#'     g <- erdos.renyi.game(tamGraph1, p1[sim])
#'     g.adj[[cont]] <- get.adjacency(g)
#'     cont <- cont+1
#' }
#' for (sim in 1:maxGraphs2) {
#'     g <- erdos.renyi.game(tamGraph2, p2[sim])
#'     g.adj[[cont]] <- get.adjacency(g)
#'     cont <- cont+1
#' }
#' for (sim in 1:maxGraphs3) {
#'     g <- erdos.renyi.game(tamGraph3, p3[sim])
#'     g.adj[[cont]] <- get.adjacency(g)
#'     cont <- cont+1
#' }
#' label <- array(0, (maxGraphs1+maxGraphs2+maxGraphs3))
#' label[1:maxGraphs1] <- 1
#' label[(maxGraphs1+1):(maxGraphs1+maxGraphs2)] <- 2
#' label[(maxGraphs1+maxGraphs2+1) : (maxGraphs1+maxGraphs2+maxGraphs2)] <- 3
#' ganova(g.adj, label, maxBoot=100)
#'
#' @export

ganova <- function(graph, labels, maxBoot=1000) {

    band <- 512
    minimos <- array(0, length(labels))
    maximos <- array(0, length(labels))

    spectrum <- list()
    for (i in 1:length(labels)) {
        tmp <- eigen(graph[[i]])$values
        spectrum[[i]] <-  tmp / sqrt(length(tmp))
        minimos[i] <- min(spectrum[[i]])
        maximos[i] <- max(spectrum[[i]])
    }

    minimo <- min(minimos)
    maximo <- max(maximos)

    densidade <- matrix(0, length(labels), band)
    bw_ <- abs(maximo - minimo) / ceiling(log2(length(spectrum[[1]])) + 1)

    for (i in 1:length(labels)) {
        tmp <- density(spectrum[[i]], bw = bw_, from=minimo, to=maximo, n=band)

        area <- trapezoidSum(tmp$x, tmp$y)
        densidade[i,] <- tmp$y/area
    }
    x_axis <- tmp$x

    media <- matrix(0, max(labels), band)
    mediaAll <- list()
    mediaAll$y <- array(0,band)
    for (j in 1:band) {
        for (i in 1:max(labels)) {
            media[i,j] <- mean(densidade[which(labels==i), j])
        }
        mediaAll$y[j] <- mean(media[,j])
    }
    mediaAll$x <- x_axis

    distOrig <- 0
    meanGroup <- list()
    meanGroup$x <- x_axis

    for(i in 1:max(labels)) {
        meanGroup$y <- media[i,]
        distOrig <- distOrig + KL(meanGroup, mediaAll) 
    }
    distOrig <- distOrig / max(labels)


    ## Permutation test
    distBoot <- array(0, maxBoot)

    for (boot in 1:maxBoot) {
        labelsB <- sample(labels, length(labels), replace=FALSE)

        mediaB <- matrix(0, max(labels), band)

        for (j in 1:band) {
            for (i in 1:max(labels)) {
                mediaB[i,j] <- mean(densidade[which(labelsB==i), j])
            }
        }

        meanBoot <- list()
        meanBoot$x <- x_axis
        for(i in 1:max(labelsB)) {
            meanBoot$y <- mediaB[i,]
            distBoot[boot] <- distBoot[boot] + KL(meanBoot, mediaAll) 
        }
        distBoot[boot] <- distBoot[boot] / max(labels)
    }

    pvalue <- length(which(distBoot >= distOrig)) / (maxBoot+1)

    return(pvalue)
}


#' Test for Association / Correlation Between Paired Samples of Graphs
#'    
#' Test for association between paired samples of graphs, using of Spearman's rho
#' correlation coefficient.
#'    
#' @param x, y lists of adjacency matrices of graphs. 'x' and 'y' must have the same length.
#'
#' @return statistic: the value of the test statistic
#' @return p.value: the p-value of the test
#' @return estimate: the estimated measure of association 'rho'
#'
#' @keywords correlation coefficient
#'    
#'    
#' @examples
#' library(igraph)
#' library(MASS)
#'
#' x <- list()
#' y <- list()
#'
#' p <- mvrnorm(100, mu=c(0,0), Sigma=matrix(c(1, 0.5, 0.5, 1), 2, 2))
#'
#' ma <- max(p)
#' mi <- min(p)
#' p[,1] <- ( p[,1] - mi )  / (ma - mi)
#' p[,2] <- ( p[,2] - mi )  / (ma - mi)
#'
#' for (i in 1:100) {
#'     x[[i]] <- get.adjacency(erdos.renyi.game(100, p[i,1]))
#'     y[[i]] <- get.adjacency(erdos.renyi.game(100, p[i,2]))
#' }
#'
#' graph.cor.test(x, y)
#'
#' @export

graph.cor.test <- function(x, y) {

    x.radius <- array(0, length(x))
    y.radius <- array(0, length(y))

    for (i in 1:length(x)) {
        x.radius[i] <- eigen(x[[i]])$values[1]
        y.radius[i] <- eigen(y[[i]])$values[1]
    }

    return(cor.test(x.radius, y.radius, method="spearman"))
}


#' Auto Correlation Function Estimation for Graphs
#'    
#' The function graph.acf computes estimates of the autocorrelation function.
#'    
#' @param x a list of adjacency matrices of graphs
#' @param plot logical. If TRUE (default) the graph.acf is plotted.
#' @return An object of class acf.
#'
#' @keywords graph radius, autocorrelation
#'    
#'    
#' @examples
#' library(stats)
#' library(igraph)
#' x <- list()
#' p <- array(0, 100)
#' p[1:3] <- rnorm(3)  
#' for (t in 4:100) {
#'     p[t] <- 0.5*p[t-3] + rnorm(1)
#' }
#' ma <- max(p)
#' mi <- min(p)
#' p <- ( p - mi )  / (ma - mi)
#' for (t in 1:100) {
#'     x[[t]] <- get.adjacency(erdos.renyi.game(100, p[t]))
#' }
#' graph.acf(x, plot=TRUE)
#'
#' @export

graph.acf <- function(x, plot=TRUE) {
    x.radius <- array(0, length(x))
    for (t in 1:length(x)) {
        x.radius[t] <- eigen(x[[t]])$values[1]
    }
    res <- acf(x.radius, plot=plot)
    return(res)
}


#' Hierarchical cluster analysis on a list of graphs.
#' @param x a list containing the adjacency matrices of the graphs
#' @param k the number of clusters
#' @param method the agglomeration method to be used. This should be 
#'               (an unambiguous abbreviation of) one of ‘"ward.D"’, ‘"ward.D2"’,
#'               ‘"single"’, ‘"complete"’, ‘"average"’ (= UPGMA), ‘"mcquitty"’
#'               (= WPGMA), ‘"median"’ (= WPGMC) or ‘"centroid"’ (= UPGMC).
#' @return hclust an object of class *hclust* which described the tree produced
#'                by the clustering process
#' @return cluster the clustering labels for each graph
#' @examples
#' require(igraph)
#' g <- list()
#' for (i in 1:10) {
#'     g[[i]] <- as.matrix(igraph::get.adjacency(
#'                         igraph::erdos.renyi.game(100, 0.5, type="gnp",
#'                         irected = FALSE)))
#' }
#' for (i in 11:20) {
#'     g[[i]] <- as.matrix(igraph::get.adjacency(
#'                         igraph::watts.strogatz.game(1, 100, 8, 0.2)))
#' }
#' for (i in 21:30) {
#'     g[[i]] <- as.matrix(igraph::get.adjacency(
#'                         igraph::barabasi.game(100, power = 1,
#'                                               directed = FALSE)))
#' }
#' graph.cluster(g, 3)
#' 
#' @export

graph.cluster <- function(x, k, method="complete") {

    mi <- array(0, length(x))
    ma <- array(0, length(x))

    spectrum <- list()
    for (i in 1:length(x)) {
        tmp <- eigen(x[[i]])$values
        spectrum[[i]] <-  tmp / sqrt(length(tmp))  ## Normalizing
        mi[i] <- min(spectrum[[i]])
        ma[i] <- max(spectrum[[i]])
    }

    minimum <- min(mi)
    maximum <- max(ma)

    densidade <- matrix(0, length(x), 512)

    for (i in 1:length(x)) {
        tmp <- density(spectrum[[i]], from=minimum, to=maximum, n=512)
        area <- trapezoidSum(tmp$x, tmp$y)
        densidade[i,] <- tmp$y/area  ## Normalizing to area equal one
    }

    d <- matrix(0, length(x), length(x))
    for (i in 1:(length(x)-1)) {
        for (j in (i+1) : length(x)) {
            d[i,j] <- d[j,i] <- JS(densidade[i,], densidade[j,])
        }
    }

    tmp <- hclust(as.dist(d), method=method)

    res <- list()
    res$hclust <- tmp
    res$cluster <- cutree(tmp, k)

    return(res)    
}


################################################################################
## Auxiliar functions
################################################################################

JS <- function(f1, f2) {
    fm <- f1
    fm$y <- (f1$y + f2$y)/2
    return((KL(f1, fm) + KL(f2, fm))/2)
}

KL <- function(f1, f2) {
    y <- f1$y
    n <- length(y)
    for (i in 1:n) {
        if (y[i] != 0 && f2$y[i] == 0)
            return(Inf)
        if (y[i] != 0)
            y[i] <- y[i]*log(y[i]/f2$y[i])
    }
    return(trapezoidSum(f1$x, y))
}


trapezoidSum <- function (x, y) {
    n <- length(x)
    delta <- (x[2] - x[1])
    area <- sum(y[2:(n-1)])
    area <- (area + (y[1] + y[n])/2)*delta
    return(area)
}

# Returns the kernel bandwidth for a given sample x
kernelBandwidth <- function(x) {
    n <- length(x)
    # Sturges' criterion
    nbins <- ceiling(log2(n) + 1)
    return(abs(max(x) - min(x))/nbins)
}
# Returns the density function for a given sample x at n points in the interval
# [form, to]
gaussianDensity <- function(x, from=NULL, to=NULL, bandwidth="Silverman", 
                            npoints=1024) {
    if (bandwidth == "Sturges")
        bw <- kernelBandwidth(x)
    else if (bandwidth == "Silverman")
        bw <- bw.nrd0(x)
    if (bw == 0)
       return(NA)
    if (is.null(from) || is.null(to))
        f <- density(x, bw=bw, n=npoints)
        #f <- density(x, n=npoints)

    else
        f <- density(x, bw=bw, from=from, to=to, n=npoints)
        #f <- density(x, from=from, to=to, n=npoints)

    area <- trapezoidSum(f$x, f$y)
    return(list("x"=f$x, "y"=f$y/area))
}

# Returns the spectral density for a given adjacency matrix A
spectralDensity <- function(A, from=NULL, to=NULL, bandwidth="Silverman", 
                            npoints=1024) {
    eigenvalues <- (as.numeric(eigen(A, only.values = TRUE)$values)/
                    sqrt(nrow(A)))
    return(gaussianDensity(eigenvalues, from, to, bandwidth=bandwidth, 
                           npoints=npoints))
}

# Returns the spectral densities for given adjacency matrices A1 and A2 at the
# same points
spectralDensities <- function(A1, A2, bandwidth="Silverman",
                              npoints=1024) {
    n1 <- nrow(A1)
    n2 <- nrow(A2)
    e1 <- (as.numeric(eigen(A1, only.values = TRUE)$values)/sqrt(n1))
    e2 <- (as.numeric(eigen(A2, only.values = TRUE)$values)/sqrt(n2))
    #b1 <- kernelBandwidth(e1)
    #b2 <- kernelBandwidth(e2)
    #from <- min(min(e1) - 3*b1, min(e2) - 3*b2)
    #to <- max(max(e1) + 3*b1, max(e2) + 3*b2)
    from <- min(e1, e2)
    to <- max(e1, e2)
    f1 <- gaussianDensity(e1, from=from, to=to, bandwidth=bandwidth, 
                          npoints=npoints)
    f2 <- gaussianDensity(e2, from=from, to=to, bandwidth=bandwidth, 
                          npoints=npoints)
    if (sum(is.na(f1)) > 0 || sum(is.na(f2)) > 0)
        return(NA)
    return(list("f1"=f1, "f2"=f2))
}

# Returns the spectral densities for a list of adjacency matrices at the
# same points
nSpectralDensities <- function (adjacencyMatrices, from=NULL, to=NULL, 
                                bandwidth="Silverman") {
    npoints <- 1024
    ngraphs <- length(adjacencyMatrices)
    n <- ncol(adjacencyMatrices[[1]])
    spectra <- matrix(NA, n, ngraphs)
    for (i in 1:ngraphs) {
        A <- adjacencyMatrices[[i]]
        eigenvalues <- (as.numeric(eigen(A, only.values = TRUE)$values)/
                       sqrt(nrow(A)))
        spectra[,i] <- eigenvalues
    }
    densities <- matrix(NA, npoints, ngraphs)
    minimum <- min(spectra)
    maximum <- max(spectra)
    if (!is.null(from) && !is.null(to)) {
        minimum <- min(minimum, from)
        maximum <- max(maximum, to)
    }
    for (i in 1:ngraphs) {
        f <- gaussianDensity(spectra[,i], bandwidth=bandwidth,
                             from=minimum, to=maximum,
                             npoints=npoints)

        densities[,i] <- f$y
        x <- f$x
    }
    return(list("x"=x, "densities"=densities))
}

# Estimates the spectral density of a graph model
modelSpectralDensity <- function(fun, n, p, ngraphs=50, from=NULL, to=NULL, 
                                 bandwidth="Silverman", npoints=1024) {
    spectra <- matrix(NA, n, ngraphs)
    for (i in 1:ngraphs) {
        A <- fun(n, p)
        eigenvalues <- (as.numeric(eigen(A, only.values = TRUE)$values)/
                       sqrt(nrow(A)))
        spectra[,i] <- eigenvalues
    }
    densities <- matrix(NA, npoints, ngraphs)
    minimum <- min(spectra)
    maximum <- max(spectra)
    if (!is.null(from) && !is.null(to)) {
        minimum <- min(minimum, from)
        maximum <- max(maximum, to)
    }
    for (i in 1:ngraphs) {
        f <- gaussianDensity(spectra[,i], from=minimum, to=maximum, 
                             bandwidth=bandwidth, npoints=npoints)

        densities[,i] <- f$y
        x <- f$x
    }
    return(list("x" = x, "y" = rowMeans(densities)))
}

matchFunction <- function(name) {
    return(match.fun(name))
}


# Watts-Strougatz graph
WSfun <- function(K){
    f <- function(n, pr) {
        WS(n, pr, K=K)
    }
    return(f)
}


# Graph models  ----------------------------------------------------------------

# Erdos-Renyi graph 
ER <- function(n, p) {
    return(as.matrix(igraph::get.adjacency(igraph::erdos.renyi.game(n, p, 
                                                                   type="gnp",
                                                            directed = FALSE))))
}

# Geometric graph
GRG <- function(n, r) {
    return(as.matrix(igraph::get.adjacency(igraph::grg.game(n, r))))
}

# Barabasi-Albert graph
BA <- function(n, ps) {
    return(as.matrix(igraph::get.adjacency(igraph::barabasi.game(n, power = ps,
                                                 directed = FALSE))))
}

# Watts-Strogatz graph
WS <- function(n, pr, K=8) {
    return(as.matrix(igraph::get.adjacency(igraph::watts.strogatz.game(1, n, K, 
                                                                       pr))))
}

# K-regular game
KR <- function(n, k) {
    return(as.matrix(get.adjacency(igraph::k.regular.game(n, k))))
}





