% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statGraph.R
\name{graph.model.selection}
\alias{graph.model.selection}
\title{Graph model selection}
\usage{
graph.model.selection(A, models = NULL, parameters = NULL,
  bandwidth = "Silverman")
}
\arguments{
\item{A}{a symmetric matrix of nonnegative real values. For unweighted graphs 
it contains only 0s and 1s and corresponds to the adjacency matrix. For 
weighted graphs, it may contain any nonnegative real value and corresponds to
the weighted adjancency matrix.}

\item{models}{either a character vector or a list of functions describing 
graph models: 

A character vector cotaining some of the following models: "ER" (Erdos-Renyi 
random graph), "GRG" (geometric random graph), "KR" (k regular random graph), 
"WS" (Watts-Strogatz model), and "BA" (Barabási-Albert model).

A list of functions. Each function returns a graph according to a graph model
and has two arguments: n (graph size) and p (a model parameter), in this 
order.

If the argument "models" is NULL, then the "ER", "WS", and "BA" models will 
be considered.}

\item{parameters}{list of numeric vectors. Each vector contains the values 
that will be considerated for the parameter of the corresponding model.
For the "ER", "GRG", "KR", "WS", and "BA" models, the parameter corresponds 
to the probability to connect a pair of vertices, the radius used to contruct 
the geometric graph in a unit square, the degree k of a regular graph, the 
probability to reconnect a vertex, and the scale exponent, respectively. For 
those models there are default parameter values that will be considered, and 
then it is optional to use this argument. However if the user provides custom 
functions to generate graphs, then it is also necessary to provide the values 
that will be considered for the parameter estimation.}

\item{bandwidth}{string indicating which criterion should be used
to choose the bandwidth for the spectral density estimation. The available 
criteria are "Silverman" (default) and "Sturges".}
}
\value{
A matrix in which each row corresponds to a model, the column "p"
corresponds to the parameter estimate, and the column "KL" corresponds to
the Kullback-Leibler divergence between the observed graph and the model in 
the row.
}
\description{
Selects the graph model that best approximates the observed graph. It can 
take a long time to execute, specially for the k regular graph model.
}
\examples{
library(igraph)
A <- as.matrix(get.adjacency(erdos.renyi.game(100, p=0.5)))

# Using the default graph models (ER, WS and BA)
result1 <- graph.model.selection(A)
result1

# Using strings to indicate the graph models
result2 <- graph.model.selection(A, models=c("ER", "WS", "BA"))
result2

# Using functions to describe the graph models
# Erdos-Renyi graph
model1 <- function(n, p) {
   return(as.matrix(get.adjacency(erdos.renyi.game(n, p))))
}
# Watts-Strougatz graph
model2 <- function(n, pr, K=8) {
    return(as.matrix(igraph::get.adjacency(igraph::watts.strogatz.game(1, n, 
                                             K, pr))))
}
# Barabasi-Albert graph
model3 <- function(n, ps) {
    return(as.matrix(igraph::get.adjacency(igraph::barabasi.game(n, power=ps,
                                                directed = FALSE))))
}
parameters <- list(seq(0, 1, 0.01), seq(0, 1, 0.01), seq(1, 4, 0.01))
result3 <- graph.model.selection(A, list(model1, model2, model3), parameters)
result3
}
\references{
Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
André Fujita. Discriminating Different Classes of Biological Networks by 
Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
(December 2012): e49949. doi:10.1371/journal.pone.0049949.
}

