% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statGraph.R
\name{graph.param.estimator}
\alias{graph.param.estimator}
\title{Graph parameter estimator}
\usage{
graph.param.estimator(A, model, parameters = NULL, bandwidth = "Silverman",
  eps = 0.01)
}
\arguments{
\item{A}{a symmetric matrix of nonnegative real values. For unweighted graphs 
it contains only 0s and 1s and corresponds to the adjacency matrix. For 
weighted graphs, it may contain any nonnegative real value and corresponds to
the weighted adjancency matrix.}

\item{model}{either a string or a function describing a graph model: 

A string that indicates one of the following models: "ER" (Erdos-Renyi random 
graph), "GRG" (geometric random graph), "KR" (k regular 
random graph), "WS" (Watts-Strogatz model), and "BA" (Barabási-Albert model).

A function that returns a graph according to a graph model. It must have two 
arguments: n (graph size) and p (a model parameter).}

\item{parameters}{numeric vector containing the values that will be 
considerated for the model parameter. For the "ER", "GRG", "KR", "WS", and 
"BA" models, the parameter corresponds to the probability to connect a pair 
of vertices, the radius used to contruct the geometric graph in a unit 
square, the degree k of a regular graph, the probability to reconnect a 
vertex, and the scale exponent, respectively. For those models there are 
default parameter values that will be considered, and then it is optional to 
use this argument. However if the user provides a custom function to generate 
graphs, then it is also necessary to provide the values that will be 
considered for the estimation.}

\item{bandwidth}{string indicating which criterion should be used
to choose the bandwidth for the spectral density estimation. The available 
criteria are "Silverman" (default) and "Sturges".}

\item{eps}{precision of the grid (default is 0.01)}
}
\value{
A list containing:
\item{p}{the parameter estimate. For the "ER", "GRG", "KR", "WS", and "BA"
models, the parameter corresponds to the probability to connect a pair of 
vertices, the radius used to contruct the geometric graph in a unit square,
the degree k of a regular graph, the probability to reconnect a vertex, and 
the scale exponent, respectively.}  
\item{KL}{the Kullback-Leibler divergence between the observed graph and the 
model obtained by the estimator.}
}
\description{
Estimates the model parameter that best approximates the model to the 
observed graph. It can take a long time to execute, specially for the k 
regular graph model.
}
\examples{
library(igraph)
A <- as.matrix(get.adjacency(erdos.renyi.game(100, p=0.5)))

# Using a string to indicate the graph model
result1 <- graph.param.estimator(A, "ER")
result1

# Using a function to describe the graph model
# Erdos-Renyi graph
model <- function(n, p) {
   return(as.matrix(get.adjacency(erdos.renyi.game(n, p))))
}
result2 <- graph.param.estimator(A, model,  seq(0, 1, 0.01))
result2

}
\references{
Daniel Yasumasa Takahashi, João Ricardo Sato, Carlos Eduardo Ferreira, and 
André Fujita. Discriminating Different Classes of Biological Networks by 
Analyzing the adjacencyMatrices Spectra Distribution. PLoS ONE 7, no. 12 
(December 2012): e49949. doi:10.1371/journal.pone.0049949.
}

